@@ -15,7 +15,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -25,6 +25,10 @@
 
 (unless (featurep 'ob-scheme)
   (signal 'missing-test-dependency "Support for Scheme code blocks"))
+(unless (featurep 'geiser)
+  (signal 'missing-test-dependency "geiser"))
+(unless (version<= "27.1" emacs-version)
+  (signal 'missing-test-dependency "Geiser required for Scheme code blocks needs Emacs >=27.1"))
 
 (ert-deftest test-ob-scheme/tables ()
   "Test table output."
